@@ -1,7 +1,7 @@
 /*    av.c
  *
- *    Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
- *    2000, 2001, 2002, 2003, 2004, 2005, 2006, by Larry Wall and others
+ *    Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000,
+ *    2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 by Larry Wall and others
  *
  *    You may distribute under the terms of either the GNU General Public
  *    License or the Artistic License, as specified in the README file.
@@ -9,8 +9,10 @@
  */
 
 /*
- * "...for the Entwives desired order, and plenty, and peace (by which they
- * meant that things should remain where they had set them)." --Treebeard
+ * '...for the Entwives desired order, and plenty, and peace (by which they
+ *  meant that things should remain where they had set them).' --Treebeard
+ *
+ *     [p.476 of _The Lord of the Rings_, III/iv: "Treebeard"]
  */
 
 /*
@@ -24,12 +26,16 @@
 void
 Perl_av_reify(pTHX_ AV *av)
 {
+    dVAR;
     I32 key;
 
+    PERL_ARGS_ASSERT_AV_REIFY;
+    assert(SvTYPE(av) == SVt_PVAV);
+
     if (AvREAL(av))
 	return;
 #ifdef DEBUGGING
-    if (SvTIED_mg((SV*)av, PERL_MAGIC_tied) && ckWARN_d(WARN_DEBUGGING))
+    if (SvTIED_mg((const SV *)av, PERL_MAGIC_tied) && ckWARN_d(WARN_DEBUGGING))
 	Perl_warner(aTHX_ packWARN(WARN_DEBUGGING), "av_reify called on tied array");
 #endif
     key = AvMAX(av) + 1;
@@ -39,7 +45,7 @@ Perl_av_reify(pTHX_ AV *av)
 	SV * const sv = AvARRAY(av)[--key];
 	assert(sv);
 	if (sv != &PL_sv_undef)
-	    (void)SvREFCNT_inc(sv);
+	    SvREFCNT_inc_simple_void_NN(sv);
     }
     key = AvARRAY(av) - AvALLOC(av);
     while (key)
@@ -60,7 +66,13 @@ extended.
 void
 Perl_av_extend(pTHX_ AV *av, I32 key)
 {
-    MAGIC * const mg = SvTIED_mg((SV*)av, PERL_MAGIC_tied);
+    dVAR;
+    MAGIC *mg;
+
+    PERL_ARGS_ASSERT_AV_EXTEND;
+    assert(SvTYPE(av) == SVt_PVAV);
+
+    mg = SvTIED_mg((const SV *)av, PERL_MAGIC_tied);
     if (mg) {
 	dSP;
 	ENTER;
@@ -68,8 +80,8 @@ Perl_av_extend(pTHX_ AV *av, I32 key)
 	PUSHSTACKi(PERLSI_MAGIC);
 	PUSHMARK(SP);
 	EXTEND(SP,2);
-	PUSHs(SvTIED_obj((SV*)av, mg));
-	PUSHs(sv_2mortal(newSViv(key+1)));
+	PUSHs(SvTIED_obj(MUTABLE_SV(av), mg));
+	mPUSHi(key + 1);
         PUTBACK;
 	call_method("EXTEND", G_SCALAR|G_DISCARD);
 	POPSTACK;
@@ -87,7 +99,7 @@ Perl_av_extend(pTHX_ AV *av, I32 key)
 	    tmp = AvARRAY(av) - AvALLOC(av);
 	    Move(AvARRAY(av), AvALLOC(av), AvFILLp(av)+1, SV*);
 	    AvMAX(av) += tmp;
-	    SvPV_set(av, (char*)AvALLOC(av));
+	    AvARRAY(av) = AvALLOC(av);
 	    if (AvREAL(av)) {
 		while (tmp)
 		    ary[--tmp] = &PL_sv_undef;
@@ -109,8 +121,22 @@ Perl_av_extend(pTHX_ AV *av, I32 key)
 		IV itmp;
 #endif
 
-#ifdef MYMALLOC
-		newmax = malloced_size((void*)AvALLOC(av))/sizeof(SV*) - 1;
+#ifdef Perl_safesysmalloc_size
+		/* Whilst it would be quite possible to move this logic around
+		   (as I did in the SV code), so as to set AvMAX(av) early,
+		   based on calling Perl_safesysmalloc_size() immediately after
+		   allocation, I'm not convinced that it is a great idea here.
+		   In an array we have to loop round setting everything to
+		   &PL_sv_undef, which means writing to memory, potentially lots
+		   of it, whereas for the SV buffer case we don't touch the
+		   "bonus" memory. So there there is no cost in telling the
+		   world about it, whereas here we have to do work before we can
+		   tell the world about it, and that work involves writing to
+		   memory that might never be read. So, I feel, better to keep
+		   the current lazy system of only writing to it if our caller
+		   has a need for more space. NWC  */
+		newmax = Perl_safesysmalloc_size((void*)AvALLOC(av)) /
+		    sizeof(const SV *) - 1;
 
 		if (key <= newmax) 
 		    goto resized;
@@ -121,25 +147,26 @@ Perl_av_extend(pTHX_ AV *av, I32 key)
 #if defined(STRANGE_MALLOC) || defined(MYMALLOC)
 		Renew(AvALLOC(av),newmax+1, SV*);
 #else
-		bytes = (newmax + 1) * sizeof(SV*);
+		bytes = (newmax + 1) * sizeof(const SV *);
 #define MALLOC_OVERHEAD 16
 		itmp = MALLOC_OVERHEAD;
 		while ((MEM_SIZE)(itmp - MALLOC_OVERHEAD) < bytes)
 		    itmp += itmp;
 		itmp -= MALLOC_OVERHEAD;
-		itmp /= sizeof(SV*);
+		itmp /= sizeof(const SV *);
 		assert(itmp > newmax);
 		newmax = itmp - 1;
 		assert(newmax >= AvMAX(av));
 		Newx(ary, newmax+1, SV*);
 		Copy(AvALLOC(av), ary, AvMAX(av)+1, SV*);
 		if (AvMAX(av) > 64)
-		    offer_nice_chunk(AvALLOC(av), (AvMAX(av)+1) * sizeof(SV*));
+		    offer_nice_chunk(AvALLOC(av),
+				     (AvMAX(av)+1) * sizeof(const SV *));
 		else
 		    Safefree(AvALLOC(av));
 		AvALLOC(av) = ary;
 #endif
-#ifdef MYMALLOC
+#ifdef Perl_safesysmalloc_size
 	      resized:
 #endif
 		ary = AvALLOC(av) + AvMAX(av) + 1;
@@ -163,7 +190,7 @@ Perl_av_extend(pTHX_ AV *av, I32 key)
 		    ary[--tmp] = &PL_sv_undef;
 	    }
 	    
-	    SvPV_set(av, (char*)AvALLOC(av));
+	    AvARRAY(av) = AvALLOC(av);
 	    AvMAX(av) = newmax;
 	}
     }
@@ -185,37 +212,39 @@ more information on how to use this function on tied arrays.
 SV**
 Perl_av_fetch(pTHX_ register AV *av, I32 key, I32 lval)
 {
-    SV *sv;
+    dVAR;
 
-    if (!av)
-	return 0;
+    PERL_ARGS_ASSERT_AV_FETCH;
+    assert(SvTYPE(av) == SVt_PVAV);
 
     if (SvRMAGICAL(av)) {
-        const MAGIC * const tied_magic = mg_find((SV*)av, PERL_MAGIC_tied);
-        if (tied_magic || mg_find((SV*)av, PERL_MAGIC_regdata)) {
-            U32 adjust_index = 1;
+        const MAGIC * const tied_magic
+	    = mg_find((const SV *)av, PERL_MAGIC_tied);
+        if (tied_magic || mg_find((const SV *)av, PERL_MAGIC_regdata)) {
+	    SV *sv;
+	    if (key < 0) {
+		I32 adjust_index = 1;
+		if (tied_magic) {
+		    /* Handle negative array indices 20020222 MJD */
+		    SV * const * const negative_indices_glob =
+			hv_fetch(SvSTASH(SvRV(SvTIED_obj(MUTABLE_SV(av),
+							 tied_magic))),
+				NEGATIVE_INDICES_VAR, 16, 0);
 
-            if (tied_magic && key < 0) {
-                /* Handle negative array indices 20020222 MJD */
-		SV * const * const negative_indices_glob =
-                    hv_fetch(SvSTASH(SvRV(SvTIED_obj((SV *)av, 
-                                                     tied_magic))), 
-                             NEGATIVE_INDICES_VAR, 16, 0);
+		    if (negative_indices_glob && SvTRUE(GvSV(*negative_indices_glob)))
+			adjust_index = 0;
+		}
 
-                if (negative_indices_glob
-                    && SvTRUE(GvSV(*negative_indices_glob)))
-                    adjust_index = 0;
-            }
-
-            if (key < 0 && adjust_index) {
-                key += AvFILL(av) + 1;
-                if (key < 0)
-                    return 0;
-            }
+		if (adjust_index) {
+		    key += AvFILL(av) + 1;
+		    if (key < 0)
+			return NULL;
+		}
+	    }
 
             sv = sv_newmortal();
 	    sv_upgrade(sv, SVt_PVLV);
-	    mg_copy((SV*)av, sv, 0, key);
+	    mg_copy(MUTABLE_SV(av), sv, 0, key);
 	    LvTYPE(sv) = 't';
 	    LvTARG(sv) = sv; /* fake (SV**) */
 	    return &(LvTARG(sv));
@@ -225,26 +254,23 @@ Perl_av_fetch(pTHX_ register AV *av, I32 key, I32 lval)
     if (key < 0) {
 	key += AvFILL(av) + 1;
 	if (key < 0)
-	    return 0;
+	    return NULL;
     }
 
     if (key > AvFILLp(av)) {
 	if (!lval)
-	    return 0;
-	sv = NEWSV(5,0);
-	return av_store(av,key,sv);
+	    return NULL;
+	return av_store(av,key,newSV(0));
     }
     if (AvARRAY(av)[key] == &PL_sv_undef) {
     emptyness:
-	if (lval) {
-	    sv = NEWSV(6,0);
-	    return av_store(av,key,sv);
-	}
-	return 0;
+	if (lval)
+	    return av_store(av,key,newSV(0));
+	return NULL;
     }
     else if (AvREIFY(av)
 	     && (!AvARRAY(av)[key]	/* eg. @_ could have freed elts */
-		 || SvTYPE(AvARRAY(av)[key]) == SVTYPEMASK)) {
+		 || SvIS_FREED(AvARRAY(av)[key]))) {
 	AvARRAY(av)[key] = &PL_sv_undef;	/* 1/2 reify */
 	goto emptyness;
     }
@@ -271,21 +297,27 @@ more information on how to use this function on tied arrays.
 SV**
 Perl_av_store(pTHX_ register AV *av, I32 key, SV *val)
 {
+    dVAR;
     SV** ary;
 
-    if (!av)
-	return 0;
+    PERL_ARGS_ASSERT_AV_STORE;
+    assert(SvTYPE(av) == SVt_PVAV);
+
+    /* S_regclass relies on being able to pass in a NULL sv
+       (unicode_alternate may be NULL).
+    */
+
     if (!val)
 	val = &PL_sv_undef;
 
     if (SvRMAGICAL(av)) {
-        const MAGIC * const tied_magic = mg_find((SV*)av, PERL_MAGIC_tied);
+        const MAGIC * const tied_magic = mg_find((const SV *)av, PERL_MAGIC_tied);
         if (tied_magic) {
             /* Handle negative array indices 20020222 MJD */
             if (key < 0) {
-                unsigned adjust_index = 1;
+		bool adjust_index = 1;
 		SV * const * const negative_indices_glob =
-                    hv_fetch(SvSTASH(SvRV(SvTIED_obj((SV *)av, 
+                    hv_fetch(SvSTASH(SvRV(SvTIED_obj(MUTABLE_SV(av), 
                                                      tied_magic))), 
                              NEGATIVE_INDICES_VAR, 16, 0);
                 if (negative_indices_glob
@@ -298,9 +330,9 @@ Perl_av_store(pTHX_ register AV *av, I32 key, SV *val)
                 }
             }
 	    if (val != &PL_sv_undef) {
-		mg_copy((SV*)av, val, 0, key);
+		mg_copy(MUTABLE_SV(av), val, 0, key);
 	    }
-	    return 0;
+	    return NULL;
         }
     }
 
@@ -308,11 +340,11 @@ Perl_av_store(pTHX_ register AV *av, I32 key, SV *val)
     if (key < 0) {
 	key += AvFILL(av) + 1;
 	if (key < 0)
-	    return 0;
+	    return NULL;
     }
 
     if (SvREADONLY(av) && key >= AvFILL(av))
-	Perl_croak(aTHX_ PL_no_modify);
+	Perl_croak(aTHX_ "%s", PL_no_modify);
 
     if (!AvREAL(av) && AvREIFY(av))
 	av_reify(av);
@@ -323,9 +355,9 @@ Perl_av_store(pTHX_ register AV *av, I32 key, SV *val)
 	if (!AvREAL(av)) {
 	    if (av == PL_curstack && key > PL_stack_sp - PL_stack_base)
 		PL_stack_sp = PL_stack_base + key;	/* XPUSH in disguise */
-	    do
+	    do {
 		ary[++AvFILLp(av)] = &PL_sv_undef;
-	    while (AvFILLp(av) < key);
+	    } while (AvFILLp(av) < key);
 	}
 	AvFILLp(av) = key;
     }
@@ -333,36 +365,18 @@ Perl_av_store(pTHX_ register AV *av, I32 key, SV *val)
 	SvREFCNT_dec(ary[key]);
     ary[key] = val;
     if (SvSMAGICAL(av)) {
+	const MAGIC* const mg = SvMAGIC(av);
 	if (val != &PL_sv_undef) {
-	    MAGIC* mg = SvMAGIC(av);
-	    sv_magic(val, (SV*)av, toLOWER(mg->mg_type), 0, key);
+	    sv_magic(val, MUTABLE_SV(av), toLOWER(mg->mg_type), 0, key);
 	}
-	mg_set((SV*)av);
+	if (PL_delaymagic && mg->mg_type == PERL_MAGIC_isa)
+	    PL_delaymagic |= DM_ARRAY;
+	else
+	   mg_set(MUTABLE_SV(av));
     }
     return &ary[key];
 }
 
-/*
-=for apidoc newAV
-
-Creates a new AV.  The reference count is set to 1.
-
-=cut
-*/
-
-AV *
-Perl_newAV(pTHX)
-{
-    register AV * const av = (AV*)NEWSV(3,0);
-
-    sv_upgrade((SV *)av, SVt_PVAV);
-    /* sv_upgrade does AvREAL_only()  */
-    AvALLOC(av) = 0;
-    SvPV_set(av, (char*)0);
-    AvMAX(av) = AvFILLp(av) = -1;
-    return av;
-}
-
 /*
 =for apidoc av_make
 
@@ -376,21 +390,21 @@ will have a reference count of 1.
 AV *
 Perl_av_make(pTHX_ register I32 size, register SV **strp)
 {
-    register AV * const av = (AV*)NEWSV(8,0);
-
-    sv_upgrade((SV *) av,SVt_PVAV);
+    register AV * const av = MUTABLE_AV(newSV_type(SVt_PVAV));
     /* sv_upgrade does AvREAL_only()  */
+    PERL_ARGS_ASSERT_AV_MAKE;
+    assert(SvTYPE(av) == SVt_PVAV);
+
     if (size) {		/* "defined" was returning undef for size==0 anyway. */
         register SV** ary;
         register I32 i;
 	Newx(ary,size,SV*);
 	AvALLOC(av) = ary;
-	SvPV_set(av, (char*)ary);
-	AvFILLp(av) = size - 1;
-	AvMAX(av) = size - 1;
+	AvARRAY(av) = ary;
+	AvFILLp(av) = AvMAX(av) = size - 1;
 	for (i = 0; i < size; i++) {
 	    assert (*strp);
-	    ary[i] = NEWSV(7,0);
+	    ary[i] = newSV(0);
 	    sv_setsv(ary[i], *strp);
 	    strp++;
 	}
@@ -398,28 +412,6 @@ Perl_av_make(pTHX_ register I32 size, register SV **strp)
     return av;
 }
 
-AV *
-Perl_av_fake(pTHX_ register I32 size, register SV **strp)
-{
-    register SV** ary;
-    register AV * const av = (AV*)NEWSV(9,0);
-
-    sv_upgrade((SV *)av, SVt_PVAV);
-    Newx(ary,size+1,SV*);
-    AvALLOC(av) = ary;
-    Copy(strp,ary,size,SV*);
-    AvFLAGS(av) = AVf_REIFY;
-    SvPV_set(av, (char*)ary);
-    AvFILLp(av) = size - 1;
-    AvMAX(av) = size - 1;
-    while (size--) {
-	assert (*strp);
-	SvTEMP_off(*strp);
-	strp++;
-    }
-    return av;
-}
-
 /*
 =for apidoc av_clear
 
@@ -432,40 +424,48 @@ array itself.
 void
 Perl_av_clear(pTHX_ register AV *av)
 {
-    register I32 key;
+    dVAR;
+    I32 extra;
+
+    PERL_ARGS_ASSERT_AV_CLEAR;
+    assert(SvTYPE(av) == SVt_PVAV);
 
 #ifdef DEBUGGING
     if (SvREFCNT(av) == 0 && ckWARN_d(WARN_DEBUGGING)) {
 	Perl_warner(aTHX_ packWARN(WARN_DEBUGGING), "Attempt to clear deleted array");
     }
 #endif
-    if (!av)
-	return;
 
     if (SvREADONLY(av))
-	Perl_croak(aTHX_ PL_no_modify);
+	Perl_croak(aTHX_ "%s", PL_no_modify);
 
     /* Give any tie a chance to cleanup first */
-    if (SvRMAGICAL(av))
-	mg_clear((SV*)av); 
+    if (SvRMAGICAL(av)) {
+	const MAGIC* const mg = SvMAGIC(av);
+	if (PL_delaymagic && mg && mg->mg_type == PERL_MAGIC_isa)
+	    PL_delaymagic |= DM_ARRAY;
+        else
+	    mg_clear(MUTABLE_SV(av)); 
+    }
 
     if (AvMAX(av) < 0)
 	return;
 
     if (AvREAL(av)) {
 	SV** const ary = AvARRAY(av);
-	key = AvFILLp(av) + 1;
-	while (key) {
-	    SV * const sv = ary[--key];
+	I32 index = AvFILLp(av) + 1;
+	while (index) {
+	    SV * const sv = ary[--index];
 	    /* undef the slot before freeing the value, because a
-	     * destructor might try to modify this arrray */
-	    ary[key] = &PL_sv_undef;
+	     * destructor might try to modify this array */
+	    ary[index] = &PL_sv_undef;
 	    SvREFCNT_dec(sv);
 	}
     }
-    if ((key = AvARRAY(av) - AvALLOC(av))) {
-	AvMAX(av) += key;
-	SvPV_set(av, (char*)AvALLOC(av));
+    extra = AvARRAY(av) - AvALLOC(av);
+    if (extra) {
+	AvMAX(av) += extra;
+	AvARRAY(av) = AvALLOC(av);
     }
     AvFILLp(av) = -1;
 
@@ -482,49 +482,53 @@ Undefines the array.  Frees the memory used by the array itself.
 void
 Perl_av_undef(pTHX_ register AV *av)
 {
-    if (!av)
-	return;
+    PERL_ARGS_ASSERT_AV_UNDEF;
+    assert(SvTYPE(av) == SVt_PVAV);
 
     /* Give any tie a chance to cleanup first */
-    if (SvTIED_mg((SV*)av, PERL_MAGIC_tied)) 
-	av_fill(av, -1);   /* mg_clear() ? */
+    if (SvTIED_mg((const SV *)av, PERL_MAGIC_tied)) 
+	av_fill(av, -1);
 
     if (AvREAL(av)) {
 	register I32 key = AvFILLp(av) + 1;
 	while (key)
 	    SvREFCNT_dec(AvARRAY(av)[--key]);
     }
+
     Safefree(AvALLOC(av));
-    AvALLOC(av) = 0;
-    SvPV_set(av, (char*)0);
+    AvALLOC(av) = NULL;
+    AvARRAY(av) = NULL;
     AvMAX(av) = AvFILLp(av) = -1;
-    /* Need to check SvMAGICAL, as during global destruction it may be that
-       AvARYLEN(av) has been freed before av, and hence the SvANY() pointer
-       is now part of the linked list of SV heads, rather than pointing to
-       the original body.  */
-    /* FIXME - audit the code for other bugs like this one.  */
-    if (AvARYLEN(av) && SvMAGICAL(AvARYLEN(av))) {
-	MAGIC *mg = mg_find (AvARYLEN(av), PERL_MAGIC_arylen);
 
-	if (mg) {
-	    /* arylen scalar holds a pointer back to the array, but doesn't
-	       own a reference. Hence the we (the array) are about to go away
-	       with it still pointing at us. Clear its pointer, else it would
-	       be pointing at free memory. See the comment in sv_magic about
-	       reference loops, and why it can't own a reference to us.  */
-	    mg->mg_obj = 0;
-	}
+    if(SvRMAGICAL(av)) mg_clear(MUTABLE_SV(av));
+}
 
-	SvREFCNT_dec(AvARYLEN(av));
-	AvARYLEN(av) = 0;
-    }
+/*
+
+=for apidoc av_create_and_push
+
+Push an SV onto the end of the array, creating the array if necessary.
+A small internal helper function to remove a commonly duplicated idiom.
+
+=cut
+*/
+
+void
+Perl_av_create_and_push(pTHX_ AV **const avp, SV *const val)
+{
+    PERL_ARGS_ASSERT_AV_CREATE_AND_PUSH;
+
+    if (!*avp)
+	*avp = newAV();
+    av_push(*avp, val);
 }
 
 /*
 =for apidoc av_push
 
 Pushes an SV onto the end of the array.  The array will grow automatically
-to accommodate the addition.
+to accommodate the addition. Like C<av_store>, this takes ownership of one
+reference count.
 
 =cut
 */
@@ -532,18 +536,21 @@ to accommodate the addition.
 void
 Perl_av_push(pTHX_ register AV *av, SV *val)
 {             
+    dVAR;
     MAGIC *mg;
-    if (!av)
-	return;
-    if (SvREADONLY(av))
-	Perl_croak(aTHX_ PL_no_modify);
 
-    if ((mg = SvTIED_mg((SV*)av, PERL_MAGIC_tied))) {
+    PERL_ARGS_ASSERT_AV_PUSH;
+    assert(SvTYPE(av) == SVt_PVAV);
+
+    if (SvREADONLY(av))
+	Perl_croak(aTHX_ "%s", PL_no_modify);
+
+    if ((mg = SvTIED_mg((const SV *)av, PERL_MAGIC_tied))) {
 	dSP;
 	PUSHSTACKi(PERLSI_MAGIC);
 	PUSHMARK(SP);
 	EXTEND(SP,2);
-	PUSHs(SvTIED_obj((SV*)av, mg));
+	PUSHs(SvTIED_obj(MUTABLE_SV(av), mg));
 	PUSHs(val);
 	PUTBACK;
 	ENTER;
@@ -567,18 +574,20 @@ is empty.
 SV *
 Perl_av_pop(pTHX_ register AV *av)
 {
+    dVAR;
     SV *retval;
     MAGIC* mg;
 
-    if (!av)
-      return &PL_sv_undef;
+    PERL_ARGS_ASSERT_AV_POP;
+    assert(SvTYPE(av) == SVt_PVAV);
+
     if (SvREADONLY(av))
-	Perl_croak(aTHX_ PL_no_modify);
-    if ((mg = SvTIED_mg((SV*)av, PERL_MAGIC_tied))) {
+	Perl_croak(aTHX_ "%s", PL_no_modify);
+    if ((mg = SvTIED_mg((const SV*)av, PERL_MAGIC_tied))) {
 	dSP;    
 	PUSHSTACKi(PERLSI_MAGIC);
 	PUSHMARK(SP);
-	XPUSHs(SvTIED_obj((SV*)av, mg));
+	XPUSHs(SvTIED_obj(MUTABLE_SV(av), mg));
 	PUTBACK;
 	ENTER;
 	if (call_method("POP", G_SCALAR)) {
@@ -595,10 +604,32 @@ Perl_av_pop(pTHX_ register AV *av)
     retval = AvARRAY(av)[AvFILLp(av)];
     AvARRAY(av)[AvFILLp(av)--] = &PL_sv_undef;
     if (SvSMAGICAL(av))
-	mg_set((SV*)av);
+	mg_set(MUTABLE_SV(av));
     return retval;
 }
 
+/*
+
+=for apidoc av_create_and_unshift_one
+
+Unshifts an SV onto the beginning of the array, creating the array if
+necessary.
+A small internal helper function to remove a commonly duplicated idiom.
+
+=cut
+*/
+
+SV **
+Perl_av_create_and_unshift_one(pTHX_ AV **const avp, SV *const val)
+{
+    PERL_ARGS_ASSERT_AV_CREATE_AND_UNSHIFT_ONE;
+
+    if (!*avp)
+	*avp = newAV();
+    av_unshift(*avp, 1);
+    return av_store(*avp, 0, val);
+}
+
 /*
 =for apidoc av_unshift
 
@@ -612,20 +643,22 @@ must then use C<av_store> to assign values to these new elements.
 void
 Perl_av_unshift(pTHX_ register AV *av, register I32 num)
 {
+    dVAR;
     register I32 i;
     MAGIC* mg;
 
-    if (!av)
-	return;
-    if (SvREADONLY(av))
-	Perl_croak(aTHX_ PL_no_modify);
+    PERL_ARGS_ASSERT_AV_UNSHIFT;
+    assert(SvTYPE(av) == SVt_PVAV);
 
-    if ((mg = SvTIED_mg((SV*)av, PERL_MAGIC_tied))) {
+    if (SvREADONLY(av))
+	Perl_croak(aTHX_ "%s", PL_no_modify);
+
+    if ((mg = SvTIED_mg((const SV *)av, PERL_MAGIC_tied))) {
 	dSP;
 	PUSHSTACKi(PERLSI_MAGIC);
 	PUSHMARK(SP);
 	EXTEND(SP,1+num);
-	PUSHs(SvTIED_obj((SV*)av, mg));
+	PUSHs(SvTIED_obj(MUTABLE_SV(av), mg));
 	while (num-- > 0) {
 	    PUSHs(&PL_sv_undef);
 	}
@@ -649,14 +682,13 @@ Perl_av_unshift(pTHX_ register AV *av, register I32 num)
     
 	AvMAX(av) += i;
 	AvFILLp(av) += i;
-	SvPV_set(av, (char*)(AvARRAY(av) - i));
+	AvARRAY(av) = AvARRAY(av) - i;
     }
     if (num) {
 	register SV **ary;
-	I32 slide;
-	i = AvFILLp(av);
+	const I32 i = AvFILLp(av);
 	/* Create extra elements */
-	slide = i > 0 ? i : 0;
+	const I32 slide = i > 0 ? i : 0;
 	num += slide;
 	av_extend(av, i + num);
 	AvFILLp(av) += num;
@@ -668,14 +700,15 @@ Perl_av_unshift(pTHX_ register AV *av, register I32 num)
 	/* Make extra elements into a buffer */
 	AvMAX(av) -= slide;
 	AvFILLp(av) -= slide;
-	SvPV_set(av, (char*)(AvARRAY(av) + slide));
+	AvARRAY(av) = AvARRAY(av) + slide;
     }
 }
 
 /*
 =for apidoc av_shift
 
-Shifts an SV off the beginning of the array.
+Shifts an SV off the beginning of the array. Returns C<&PL_sv_undef> if the 
+array is empty.
 
 =cut
 */
@@ -683,18 +716,20 @@ Shifts an SV off the beginning of the array.
 SV *
 Perl_av_shift(pTHX_ register AV *av)
 {
+    dVAR;
     SV *retval;
     MAGIC* mg;
 
-    if (!av)
-	return &PL_sv_undef;
+    PERL_ARGS_ASSERT_AV_SHIFT;
+    assert(SvTYPE(av) == SVt_PVAV);
+
     if (SvREADONLY(av))
-	Perl_croak(aTHX_ PL_no_modify);
-    if ((mg = SvTIED_mg((SV*)av, PERL_MAGIC_tied))) {
+	Perl_croak(aTHX_ "%s", PL_no_modify);
+    if ((mg = SvTIED_mg((const SV*)av, PERL_MAGIC_tied))) {
 	dSP;
 	PUSHSTACKi(PERLSI_MAGIC);
 	PUSHMARK(SP);
-	XPUSHs(SvTIED_obj((SV*)av, mg));
+	XPUSHs(SvTIED_obj(MUTABLE_SV(av), mg));
 	PUTBACK;
 	ENTER;
 	if (call_method("SHIFT", G_SCALAR)) {
@@ -711,54 +746,66 @@ Perl_av_shift(pTHX_ register AV *av)
     retval = *AvARRAY(av);
     if (AvREAL(av))
 	*AvARRAY(av) = &PL_sv_undef;
-    SvPV_set(av, (char*)(AvARRAY(av) + 1));
+    AvARRAY(av) = AvARRAY(av) + 1;
     AvMAX(av)--;
     AvFILLp(av)--;
     if (SvSMAGICAL(av))
-	mg_set((SV*)av);
+	mg_set(MUTABLE_SV(av));
     return retval;
 }
 
 /*
 =for apidoc av_len
 
-Returns the highest index in the array.  Returns -1 if the array is
-empty.
+Returns the highest index in the array.  The number of elements in the
+array is C<av_len(av) + 1>.  Returns -1 if the array is empty.
 
 =cut
 */
 
 I32
-Perl_av_len(pTHX_ register AV *av)
+Perl_av_len(pTHX_ register const AV *av)
 {
+    PERL_ARGS_ASSERT_AV_LEN;
+    assert(SvTYPE(av) == SVt_PVAV);
+
     return AvFILL(av);
 }
 
 /*
 =for apidoc av_fill
 
-Ensure than an array has a given number of elements, equivalent to
+Set the highest index in the array to the given number, equivalent to
 Perl's C<$#array = $fill;>.
 
+The number of elements in the an array will be C<fill + 1> after
+av_fill() returns.  If the array was previously shorter then the
+additional elements appended are set to C<PL_sv_undef>.  If the array
+was longer, then the excess elements are freed.  C<av_fill(av, -1)> is
+the same as C<av_clear(av)>.
+
 =cut
 */
 void
 Perl_av_fill(pTHX_ register AV *av, I32 fill)
 {
+    dVAR;
     MAGIC *mg;
-    if (!av)
-	Perl_croak(aTHX_ "panic: null array");
+
+    PERL_ARGS_ASSERT_AV_FILL;
+    assert(SvTYPE(av) == SVt_PVAV);
+
     if (fill < 0)
 	fill = -1;
-    if ((mg = SvTIED_mg((SV*)av, PERL_MAGIC_tied))) {
+    if ((mg = SvTIED_mg((const SV *)av, PERL_MAGIC_tied))) {
 	dSP;            
 	ENTER;
 	SAVETMPS;
 	PUSHSTACKi(PERLSI_MAGIC);
 	PUSHMARK(SP);
 	EXTEND(SP,2);
-	PUSHs(SvTIED_obj((SV*)av, mg));
-	PUSHs(sv_2mortal(newSViv(fill+1)));
+	PUSHs(SvTIED_obj(MUTABLE_SV(av), mg));
+	mPUSHi(fill + 1);
 	PUTBACK;
 	call_method("STORESIZE", G_SCALAR|G_DISCARD);
 	POPSTACK;
@@ -768,7 +815,7 @@ Perl_av_fill(pTHX_ register AV *av, I32 fill)
     }
     if (fill <= AvMAX(av)) {
 	I32 key = AvFILLp(av);
-	SV** ary = AvARRAY(av);
+	SV** const ary = AvARRAY(av);
 
 	if (AvREAL(av)) {
 	    while (key > fill) {
@@ -783,7 +830,7 @@ Perl_av_fill(pTHX_ register AV *av, I32 fill)
 	    
 	AvFILLp(av) = fill;
 	if (SvSMAGICAL(av))
-	    mg_set((SV*)av);
+	    mg_set(MUTABLE_SV(av));
     }
     else
 	(void)av_store(av,fill,&PL_sv_undef);
@@ -801,23 +848,26 @@ and null is returned.
 SV *
 Perl_av_delete(pTHX_ AV *av, I32 key, I32 flags)
 {
+    dVAR;
     SV *sv;
 
-    if (!av)
-	return Nullsv;
+    PERL_ARGS_ASSERT_AV_DELETE;
+    assert(SvTYPE(av) == SVt_PVAV);
+
     if (SvREADONLY(av))
-	Perl_croak(aTHX_ PL_no_modify);
+	Perl_croak(aTHX_ "%s", PL_no_modify);
 
     if (SvRMAGICAL(av)) {
-        const MAGIC * const tied_magic = mg_find((SV*)av, PERL_MAGIC_tied);
-        if ((tied_magic || mg_find((SV*)av, PERL_MAGIC_regdata))) {
+        const MAGIC * const tied_magic
+	    = mg_find((const SV *)av, PERL_MAGIC_tied);
+        if ((tied_magic || mg_find((const SV *)av, PERL_MAGIC_regdata))) {
             /* Handle negative array indices 20020222 MJD */
             SV **svp;
             if (key < 0) {
                 unsigned adjust_index = 1;
                 if (tied_magic) {
 		    SV * const * const negative_indices_glob =
-                        hv_fetch(SvSTASH(SvRV(SvTIED_obj((SV *)av, 
+                        hv_fetch(SvSTASH(SvRV(SvTIED_obj(MUTABLE_SV(av), 
                                                          tied_magic))), 
                                  NEGATIVE_INDICES_VAR, 16, 0);
                     if (negative_indices_glob
@@ -827,7 +877,7 @@ Perl_av_delete(pTHX_ AV *av, I32 key, I32 flags)
                 if (adjust_index) {
                     key += AvFILL(av) + 1;
                     if (key < 0)
-                        return Nullsv;
+			return NULL;
                 }
             }
             svp = av_fetch(av, key, TRUE);
@@ -838,7 +888,7 @@ Perl_av_delete(pTHX_ AV *av, I32 key, I32 flags)
                     sv_unmagic(sv, PERL_MAGIC_tiedelem); /* No longer an element */
                     return sv;
                 }
-                return Nullsv;     
+		return NULL;
             }
         }
     }
@@ -846,11 +896,11 @@ Perl_av_delete(pTHX_ AV *av, I32 key, I32 flags)
     if (key < 0) {
 	key += AvFILL(av) + 1;
 	if (key < 0)
-	    return Nullsv;
+	    return NULL;
     }
 
     if (key > AvFILLp(av))
-	return Nullsv;
+	return NULL;
     else {
 	if (!AvREAL(av) && AvREIFY(av))
 	    av_reify(av);
@@ -864,11 +914,11 @@ Perl_av_delete(pTHX_ AV *av, I32 key, I32 flags)
 	else
 	    AvARRAY(av)[key] = &PL_sv_undef;
 	if (SvSMAGICAL(av))
-	    mg_set((SV*)av);
+	    mg_set(MUTABLE_SV(av));
     }
     if (flags & G_DISCARD) {
 	SvREFCNT_dec(sv);
-	sv = Nullsv;
+	sv = NULL;
     }
     else if (AvREAL(av))
 	sv = sv_2mortal(sv);
@@ -888,21 +938,22 @@ C<&PL_sv_undef>.
 bool
 Perl_av_exists(pTHX_ AV *av, I32 key)
 {
-    if (!av)
-	return FALSE;
-
+    dVAR;
+    PERL_ARGS_ASSERT_AV_EXISTS;
+    assert(SvTYPE(av) == SVt_PVAV);
 
     if (SvRMAGICAL(av)) {
-        const MAGIC * const tied_magic = mg_find((SV*)av, PERL_MAGIC_tied);
-        if (tied_magic || mg_find((SV*)av, PERL_MAGIC_regdata)) {
-            SV *sv = sv_newmortal();
+        const MAGIC * const tied_magic
+	    = mg_find((const SV *)av, PERL_MAGIC_tied);
+        if (tied_magic || mg_find((const SV *)av, PERL_MAGIC_regdata)) {
+	    SV * const sv = sv_newmortal();
             MAGIC *mg;
             /* Handle negative array indices 20020222 MJD */
             if (key < 0) {
                 unsigned adjust_index = 1;
                 if (tied_magic) {
 		    SV * const * const negative_indices_glob =
-                        hv_fetch(SvSTASH(SvRV(SvTIED_obj((SV *)av, 
+                        hv_fetch(SvSTASH(SvRV(SvTIED_obj(MUTABLE_SV(av), 
                                                          tied_magic))), 
                                  NEGATIVE_INDICES_VAR, 16, 0);
                     if (negative_indices_glob
@@ -916,7 +967,7 @@ Perl_av_exists(pTHX_ AV *av, I32 key)
                 }
             }
 
-            mg_copy((SV*)av, sv, 0, key);
+            mg_copy(MUTABLE_SV(av), sv, 0, key);
             mg = mg_find(sv, PERL_MAGIC_tiedelem);
             if (mg) {
                 magic_existspack(sv, mg);
@@ -941,108 +992,24 @@ Perl_av_exists(pTHX_ AV *av, I32 key)
 	return FALSE;
 }
 
-/* AVHV: Support for treating arrays as if they were hashes.  The
- * first element of the array should be a hash reference that maps
- * hash keys to array indices.
- */
+SV **
+Perl_av_arylen_p(pTHX_ AV *av) {
+    dVAR;
+    MAGIC *mg;
 
-STATIC I32
-S_avhv_index_sv(pTHX_ SV* sv)
-{
-    I32 index = SvIV(sv);
-    if (index < 1)
-	Perl_croak(aTHX_ "Bad index while coercing array into hash");
-    return index;    
-}
+    PERL_ARGS_ASSERT_AV_ARYLEN_P;
+    assert(SvTYPE(av) == SVt_PVAV);
 
-STATIC I32
-S_avhv_index(pTHX_ AV *av, SV *keysv, U32 hash)
-{
-    HV *keys;
-    HE *he;
-    STRLEN n_a;
+    mg = mg_find((const SV *)av, PERL_MAGIC_arylen_p);
 
-    keys = avhv_keys(av);
-    he = hv_fetch_ent(keys, keysv, FALSE, hash);
-    if (!he)
-        Perl_croak(aTHX_ "No such pseudo-hash field \"%s\"", SvPV(keysv,n_a));
-    return avhv_index_sv(HeVAL(he));
-}
-
-HV*
-Perl_avhv_keys(pTHX_ AV *av)
-{
-    SV **keysp = av_fetch(av, 0, FALSE);
-    if (keysp) {
-	SV *sv = *keysp;
-	if (SvGMAGICAL(sv))
-	    mg_get(sv);
-	if (SvROK(sv)) {
-            if (ckWARN(WARN_DEPRECATED) && !sv_isa(sv, "pseudohash"))
-	        Perl_warner(aTHX_ packWARN(WARN_DEPRECATED),
-			    "Pseudo-hashes are deprecated");
-	    sv = SvRV(sv);
-	    if (SvTYPE(sv) == SVt_PVHV)
-		return (HV*)sv;
-	}
+    if (!mg) {
+	mg = sv_magicext(MUTABLE_SV(av), 0, PERL_MAGIC_arylen_p,
+			 &PL_vtbl_arylen_p, 0, 0);
+	assert(mg);
+	/* sv_magicext won't set this for us because we pass in a NULL obj  */
+	mg->mg_flags |= MGf_REFCOUNTED;
     }
-    Perl_croak(aTHX_ "Can't coerce array into hash");
-    return Nullhv;
-}
-
-SV**
-Perl_avhv_store_ent(pTHX_ AV *av, SV *keysv, SV *val, U32 hash)
-{
-    return av_store(av, avhv_index(av, keysv, hash), val);
-}
-
-SV**
-Perl_avhv_fetch_ent(pTHX_ AV *av, SV *keysv, I32 lval, U32 hash)
-{
-    return av_fetch(av, avhv_index(av, keysv, hash), lval);
-}
-
-SV *
-Perl_avhv_delete_ent(pTHX_ AV *av, SV *keysv, I32 flags, U32 hash)
-{
-    HV *keys = avhv_keys(av);
-    HE *he;
-	
-    he = hv_fetch_ent(keys, keysv, FALSE, hash);
-    if (!he || !SvOK(HeVAL(he)))
-	return Nullsv;
-
-    return av_delete(av, avhv_index_sv(HeVAL(he)), flags);
-}
-
-/* Check for the existence of an element named by a given key.
- *
- */
-bool
-Perl_avhv_exists_ent(pTHX_ AV *av, SV *keysv, U32 hash)
-{
-    HV *keys = avhv_keys(av);
-    HE *he;
-	
-    he = hv_fetch_ent(keys, keysv, FALSE, hash);
-    if (!he || !SvOK(HeVAL(he)))
-	return FALSE;
-
-    return av_exists(av, avhv_index_sv(HeVAL(he)));
-}
-
-HE *
-Perl_avhv_iternext(pTHX_ AV *av)
-{
-    HV *keys = avhv_keys(av);
-    return hv_iternext(keys);
-}
-
-SV *
-Perl_avhv_iterval(pTHX_ AV *av, register HE *entry)
-{
-    SV *sv = hv_iterval(avhv_keys(av), entry);
-    return *av_fetch(av, avhv_index_sv(sv), TRUE);
+    return &(mg->mg_obj);
 }
 
 /*
