@@ -1,9 +1,23 @@
 #!/usr/bin/perl
+# 
+# Regenerate (overwriting only if changed):
+#
+#    lib/warnings.pm
+#    warnings.h
+#
+# from information hardcoded into this script (the $tree hash), plus the
+# template for warnings.pm in the DATA section.
+#
+# With an argument of 'tree', just dump the contents of $tree and exits.
+# Also accepts the standard regen_lib -q and -v args.
+#
+# This script is normally invoked from regen.pl.
 
-$VERSION = '1.02';
+$VERSION = '1.02_02';
 
 BEGIN {
-  push @INC, './lib';
+    require 'regen_lib.pl';
+    push @INC, './lib';
 }
 use strict ;
 
@@ -49,7 +63,6 @@ my $tree = {
        	'misc'		=> [ 5.008, DEFAULT_OFF],
        	'regexp'	=> [ 5.008, DEFAULT_OFF],
        	'glob'		=> [ 5.008, DEFAULT_OFF],
-       	'y2k'		=> [ 5.008, DEFAULT_OFF],
        	'untie'		=> [ 5.008, DEFAULT_OFF],
 	'substr'	=> [ 5.008, DEFAULT_OFF],
 	'taint'		=> [ 5.008, DEFAULT_OFF],
@@ -62,7 +75,6 @@ my $tree = {
        	'pack'		=> [ 5.008, DEFAULT_OFF],
        	'unpack'	=> [ 5.008, DEFAULT_OFF],
        	'threads'	=> [ 5.008, DEFAULT_OFF],
-	'assertions'    => [ 5.009, DEFAULT_OFF],
 
        	 #'default'	=> [ 5.008, DEFAULT_ON ],
   	}],
@@ -250,13 +262,12 @@ if (@ARGV && $ARGV[0] eq "tree")
     exit ;
 }
 
-unlink "warnings.h";
-unlink "lib/warnings.pm";
-open(WARN, ">warnings.h") || die "Can't create warnings.h: $!\n";
-open(PM, ">lib/warnings.pm") || die "Can't create lib/warnings.pm: $!\n";
+my $warn = safer_open("warnings.h-new");
+my $pm = safer_open("lib/warnings.pm-new");
 
-print WARN <<'EOM' ;
-/* !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
+print $warn <<'EOM' ;
+/* -*- buffer-read-only: t -*-
+   !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
    This file is built by warnings.pl
    Any changes made here will be lost!
 */
@@ -274,12 +285,15 @@ print WARN <<'EOM' ;
 #define G_WARN_ONCE		8	/* set if 'once' ever enabled */
 #define G_WARN_ALL_MASK		(G_WARN_ALL_ON|G_WARN_ALL_OFF)
 
-#define pWARN_STD		Nullsv
-#define pWARN_ALL		(Nullsv+1)	/* use warnings 'all' */
-#define pWARN_NONE		(Nullsv+2)	/* no  warnings 'all' */
+#define pWARN_STD		NULL
+#define pWARN_ALL		(((STRLEN*)0)+1)    /* use warnings 'all' */
+#define pWARN_NONE		(((STRLEN*)0)+2)    /* no  warnings 'all' */
 
 #define specialWARN(x)		((x) == pWARN_STD || (x) == pWARN_ALL ||	\
 				 (x) == pWARN_NONE)
+
+/* if PL_warnhook is set to this value, then warnings die */
+#define PERL_WARNHOOK_FATAL	(&PL_sv_placeholder)
 EOM
 
 my $offset = 0 ;
@@ -303,89 +317,45 @@ my $k ;
 my $last_ver = 0;
 foreach $k (sort { $a <=> $b } keys %ValueToName) {
     my ($name, $version) = @{ $ValueToName{$k} };
-    print WARN "\n/* Warnings Categories added in Perl $version */\n\n"
+    print $warn "\n/* Warnings Categories added in Perl $version */\n\n"
         if $last_ver != $version ;
-    print WARN tab(5, "#define WARN_$name"), "$k\n" ;
+    print $warn tab(5, "#define WARN_$name"), "$k\n" ;
     $last_ver = $version ;
 }
-print WARN "\n" ;
+print $warn "\n" ;
 
-print WARN tab(5, '#define WARNsize'),	"$warn_size\n" ;
+print $warn tab(5, '#define WARNsize'),	"$warn_size\n" ;
 #print WARN tab(5, '#define WARN_ALLstring'), '"', ('\377' x $warn_size) , "\"\n" ;
-print WARN tab(5, '#define WARN_ALLstring'), '"', ('\125' x $warn_size) , "\"\n" ;
-print WARN tab(5, '#define WARN_NONEstring'), '"', ('\0' x $warn_size) , "\"\n" ;
-my $WARN_TAINTstring = mkOct($warn_size, map $_ * 2, @{ $list{'taint'} });
+print $warn tab(5, '#define WARN_ALLstring'), '"', ('\125' x $warn_size) , "\"\n" ;
+print $warn tab(5, '#define WARN_NONEstring'), '"', ('\0' x $warn_size) , "\"\n" ;
 
-print WARN tab(5, '#define WARN_TAINTstring'), qq["$WARN_TAINTstring"\n] ;
-
-print WARN <<'EOM';
+print $warn <<'EOM';
 
 #define isLEXWARN_on 	(PL_curcop->cop_warnings != pWARN_STD)
 #define isLEXWARN_off	(PL_curcop->cop_warnings == pWARN_STD)
 #define isWARN_ONCE	(PL_dowarn & (G_WARN_ON|G_WARN_ONCE))
-#define isWARN_on(c,x)	(IsSet(SvPVX(c), 2*(x)))
-#define isWARNf_on(c,x)	(IsSet(SvPVX(c), 2*(x)+1))
+#define isWARN_on(c,x)	(IsSet((U8 *)(c + 1), 2*(x)))
+#define isWARNf_on(c,x)	(IsSet((U8 *)(c + 1), 2*(x)+1))
 
-#define ckWARN(x)							\
-	( (isLEXWARN_on && PL_curcop->cop_warnings != pWARN_NONE &&	\
-	      (PL_curcop->cop_warnings == pWARN_ALL ||			\
-	       isWARN_on(PL_curcop->cop_warnings, x) ) )		\
-	  || (isLEXWARN_off && PL_dowarn & G_WARN_ON) )
+#define DUP_WARNINGS(p)		\
+    (specialWARN(p) ? (STRLEN*)(p)	\
+    : (STRLEN*)CopyD(p, PerlMemShared_malloc(sizeof(*p)+*p), sizeof(*p)+*p, \
+		     			     char))
 
-#define ckWARN2(x,y)							\
-	  ( (isLEXWARN_on && PL_curcop->cop_warnings != pWARN_NONE &&	\
-	      (PL_curcop->cop_warnings == pWARN_ALL ||			\
-	        isWARN_on(PL_curcop->cop_warnings, x)  ||		\
-	        isWARN_on(PL_curcop->cop_warnings, y) ) ) 		\
-	    ||	(isLEXWARN_off && PL_dowarn & G_WARN_ON) )
+#define ckWARN(w)		Perl_ckwarn(aTHX_ packWARN(w))
+#define ckWARN2(w1,w2)		Perl_ckwarn(aTHX_ packWARN2(w1,w2))
+#define ckWARN3(w1,w2,w3)	Perl_ckwarn(aTHX_ packWARN3(w1,w2,w3))
+#define ckWARN4(w1,w2,w3,w4)	Perl_ckwarn(aTHX_ packWARN4(w1,w2,w3,w4))
 
-#define ckWARN3(x,y,z)							\
-	  ( (isLEXWARN_on && PL_curcop->cop_warnings != pWARN_NONE &&	\
-	      (PL_curcop->cop_warnings == pWARN_ALL ||			\
-	        isWARN_on(PL_curcop->cop_warnings, x)  ||		\
-	        isWARN_on(PL_curcop->cop_warnings, y)  ||		\
-	        isWARN_on(PL_curcop->cop_warnings, z) ) ) 		\
-	    ||	(isLEXWARN_off && PL_dowarn & G_WARN_ON) )
+#define ckWARN_d(w)		Perl_ckwarn_d(aTHX_ packWARN(w))
+#define ckWARN2_d(w1,w2)	Perl_ckwarn_d(aTHX_ packWARN2(w1,w2))
+#define ckWARN3_d(w1,w2,w3)	Perl_ckwarn_d(aTHX_ packWARN3(w1,w2,w3))
+#define ckWARN4_d(w1,w2,w3,w4)	Perl_ckwarn_d(aTHX_ packWARN4(w1,w2,w3,w4))
 
-#define ckWARN4(x,y,z,t)						\
-	  ( (isLEXWARN_on && PL_curcop->cop_warnings != pWARN_NONE &&	\
-	      (PL_curcop->cop_warnings == pWARN_ALL ||			\
-	        isWARN_on(PL_curcop->cop_warnings, x)  ||		\
-	        isWARN_on(PL_curcop->cop_warnings, y)  ||		\
-	        isWARN_on(PL_curcop->cop_warnings, z)  ||		\
-	        isWARN_on(PL_curcop->cop_warnings, t) ) ) 		\
-	    ||	(isLEXWARN_off && PL_dowarn & G_WARN_ON) )
-
-#define ckWARN_d(x)							\
-	  (isLEXWARN_off || PL_curcop->cop_warnings == pWARN_ALL ||	\
-	     (PL_curcop->cop_warnings != pWARN_NONE &&			\
-	      isWARN_on(PL_curcop->cop_warnings, x) ) )
-
-#define ckWARN2_d(x,y)							\
-	  (isLEXWARN_off || PL_curcop->cop_warnings == pWARN_ALL ||	\
-	     (PL_curcop->cop_warnings != pWARN_NONE &&			\
-	        (isWARN_on(PL_curcop->cop_warnings, x)  ||		\
-	         isWARN_on(PL_curcop->cop_warnings, y) ) ) )
-
-#define ckWARN3_d(x,y,z)						\
-	  (isLEXWARN_off || PL_curcop->cop_warnings == pWARN_ALL ||	\
-	     (PL_curcop->cop_warnings != pWARN_NONE &&			\
-	        (isWARN_on(PL_curcop->cop_warnings, x)  ||		\
-	         isWARN_on(PL_curcop->cop_warnings, y)  ||		\
-	         isWARN_on(PL_curcop->cop_warnings, z) ) ) )
-
-#define ckWARN4_d(x,y,z,t)						\
-	  (isLEXWARN_off || PL_curcop->cop_warnings == pWARN_ALL ||	\
-	     (PL_curcop->cop_warnings != pWARN_NONE &&			\
-	        (isWARN_on(PL_curcop->cop_warnings, x)  ||		\
-	         isWARN_on(PL_curcop->cop_warnings, y)  ||		\
-	         isWARN_on(PL_curcop->cop_warnings, z)  ||		\
-	         isWARN_on(PL_curcop->cop_warnings, t) ) ) )
-
-#define packWARN(a)		(a                                 )
-#define packWARN2(a,b)		((a) | (b)<<8                      )
-#define packWARN3(a,b,c)	((a) | (b)<<8 | (c) <<16           )
-#define packWARN4(a,b,c,d)	((a) | (b)<<8 | (c) <<16 | (d) <<24)
+#define packWARN(a)		(a                                      )
+#define packWARN2(a,b)		((a) | ((b)<<8)                         )
+#define packWARN3(a,b,c)	((a) | ((b)<<8) | ((c)<<16)             )
+#define packWARN4(a,b,c,d)	((a) | ((b)<<8) | ((c)<<16) | ((d) <<24))
 
 #define unpackWARN1(x)		((x)        & 0xFF)
 #define unpackWARN2(x)		(((x) >>8)  & 0xFF)
@@ -401,73 +371,76 @@ print WARN <<'EOM';
 	      isWARNf_on(PL_curcop->cop_warnings, unpackWARN4(x))))
 
 /* end of file warnings.h */
-
+/* ex: set ro: */
 EOM
 
-close WARN ;
+safer_close $warn;
+rename_if_different("warnings.h-new", "warnings.h");
 
 while (<DATA>) {
     last if /^KEYWORDS$/ ;
-    print PM $_ ;
+    print $pm $_ ;
 }
 
 #$list{'all'} = [ $offset .. 8 * ($warn_size/2) - 1 ] ;
 
 $last_ver = 0;
-print PM "our %Offsets : unique = (\n" ;
+print $pm "our %Offsets = (\n" ;
 foreach my $k (sort { $a <=> $b } keys %ValueToName) {
     my ($name, $version) = @{ $ValueToName{$k} };
     $name = lc $name;
     $k *= 2 ;
     if ( $last_ver != $version ) {
-        print PM "\n";
-        print PM tab(4, "    # Warnings Categories added in Perl $version");
-        print PM "\n\n";
+        print $pm "\n";
+        print $pm tab(4, "    # Warnings Categories added in Perl $version");
+        print $pm "\n\n";
     }
-    print PM tab(4, "    '$name'"), "=> $k,\n" ;
+    print $pm tab(4, "    '$name'"), "=> $k,\n" ;
     $last_ver = $version;
 }
 
-print PM "  );\n\n" ;
+print $pm "  );\n\n" ;
 
-print PM "our %Bits : unique = (\n" ;
+print $pm "our %Bits = (\n" ;
 foreach $k (sort keys  %list) {
 
     my $v = $list{$k} ;
     my @list = sort { $a <=> $b } @$v ;
 
-    print PM tab(4, "    '$k'"), '=> "',
+    print $pm tab(4, "    '$k'"), '=> "',
 		# mkHex($warn_size, @list),
 		mkHex($warn_size, map $_ * 2 , @list),
 		'", # [', mkRange(@list), "]\n" ;
 }
 
-print PM "  );\n\n" ;
+print $pm "  );\n\n" ;
 
-print PM "our %DeadBits : unique = (\n" ;
+print $pm "our %DeadBits = (\n" ;
 foreach $k (sort keys  %list) {
 
     my $v = $list{$k} ;
     my @list = sort { $a <=> $b } @$v ;
 
-    print PM tab(4, "    '$k'"), '=> "',
+    print $pm tab(4, "    '$k'"), '=> "',
 		# mkHex($warn_size, @list),
 		mkHex($warn_size, map $_ * 2 + 1 , @list),
 		'", # [', mkRange(@list), "]\n" ;
 }
 
-print PM "  );\n\n" ;
-print PM '$NONE     = "', ('\0' x $warn_size) , "\";\n" ;
-print PM '$LAST_BIT = ' . "$index ;\n" ;
-print PM '$BYTES    = ' . "$warn_size ;\n" ;
+print $pm "  );\n\n" ;
+print $pm '$NONE     = "', ('\0' x $warn_size) , "\";\n" ;
+print $pm '$LAST_BIT = ' . "$index ;\n" ;
+print $pm '$BYTES    = ' . "$warn_size ;\n" ;
 while (<DATA>) {
-    print PM $_ ;
+    print $pm $_ ;
 }
 
-close PM ;
+print $pm "# ex: set ro:\n";
+safer_close $pm;
+rename_if_different("lib/warnings.pm-new", "lib/warnings.pm");
 
 __END__
-
+# -*- buffer-read-only: t -*-
 # !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
 # This file was created by warnings.pl
 # Any changes made here will be lost.
@@ -475,7 +448,14 @@ __END__
 
 package warnings;
 
-our $VERSION = '1.03';
+our $VERSION = '1.06';
+
+# Verify that we're called correctly so that warnings will work.
+# see also strict.pm.
+unless ( __FILE__ =~ /(^|[\/\\])\Q${\__PACKAGE__}\E\.pmc?$/ ) {
+    my (undef, $f, $l) = caller;
+    die("Incorrect use of pragma '${\__PACKAGE__}' at $f line $l.\n");
+}
 
 =head1 NAME
 
@@ -600,14 +580,14 @@ See L<perlmodlib/Pragmatic Modules> and L<perllexwarn>.
 
 =cut
 
-use Carp ();
-
 KEYWORDS
 
 $All = "" ; vec($All, $Offsets{'all'}, 2) = 3 ;
 
 sub Croaker
 {
+    require Carp::Heavy; # this initializes %CarpInternal
+    local $Carp::CarpInternal{'warnings'};
     delete $Carp::CarpInternal{'warnings'};
     Carp::croak(@_);
 }
@@ -710,6 +690,8 @@ sub unimport
     ${^WARNING_BITS} = $mask ;
 }
 
+my %builtin_type; @builtin_type{qw(SCALAR ARRAY HASH CODE REF GLOB LVALUE Regexp)} = ();
+
 sub __chk
 {
     my $category ;
@@ -719,10 +701,10 @@ sub __chk
     if (@_) {
         # check the category supplied.
         $category = shift ;
-        if (ref $category) {
-            Croaker ("not an object")
-                if $category !~ /^([^=]+)=/ ;
-	    $category = $1 ;
+        if (my $type = ref $category) {
+            Croaker("not an object")
+                if exists $builtin_type{$type};
+	    $category = $type;
             $isobj = 1 ;
         }
         $offset = $Offsets{$category};
@@ -779,6 +761,7 @@ sub warn
 
     my $message = pop ;
     my ($callers_bitmask, $offset, $i) = __chk(@_) ;
+    require Carp;
     Carp::croak($message)
 	if vec($callers_bitmask, $offset+1, 1) ||
 	   vec($callers_bitmask, $Offsets{'all'}+1, 1) ;
@@ -798,6 +781,7 @@ sub warnif
             	(vec($callers_bitmask, $offset, 1) ||
             	vec($callers_bitmask, $Offsets{'all'}, 1)) ;
 
+    require Carp;
     Carp::croak($message)
 	if vec($callers_bitmask, $offset+1, 1) ||
 	   vec($callers_bitmask, $Offsets{'all'}+1, 1) ;
