@@ -9,8 +9,10 @@
  */
 
 /*
- * "Perilous to us all are the devices of an art deeper than we possess
- * ourselves." --Gandalf
+ * 'Perilous to us all are the devices of an art deeper than we possess
+ *  ourselves.'                                            --Gandalf
+ *
+ *     [p.597 of _The Lord of the Rings_, III/xi: "The Palantr"]
  */
 
 
@@ -47,7 +49,7 @@ static const char file[] = __FILE__;
 void
 Perl_boot_core_xsutils(pTHX)
 {
-    newXS("attributes::bootstrap", XS_attributes_bootstrap, (char *)file);
+    newXS("attributes::bootstrap",	XS_attributes_bootstrap,	file);
 }
 
 #include "XSUB.h"
@@ -55,6 +57,7 @@ Perl_boot_core_xsutils(pTHX)
 static int
 modify_SV_attributes(pTHX_ SV *sv, SV **retlist, SV **attrlist, int numattrs)
 {
+    dVAR;
     SV *attr;
     int nret;
 
@@ -72,32 +75,30 @@ modify_SV_attributes(pTHX_ SV *sv, SV **retlist, SV **attrlist, int numattrs)
 	    switch ((int)len) {
 	    case 6:
 		switch (name[3]) {
-#ifdef CVf_LVALUE
 		case 'l':
 		    if (memEQ(name, "lvalue", 6)) {
 			if (negated)
-			    CvFLAGS((CV*)sv) &= ~CVf_LVALUE;
+			    CvFLAGS(MUTABLE_CV(sv)) &= ~CVf_LVALUE;
 			else
-			    CvFLAGS((CV*)sv) |= CVf_LVALUE;
+			    CvFLAGS(MUTABLE_CV(sv)) |= CVf_LVALUE;
 			continue;
 		    }
 		    break;
-#endif
 		case 'k':
 		    if (memEQ(name, "locked", 6)) {
 			if (negated)
-			    CvFLAGS((CV*)sv) &= ~CVf_LOCKED;
+			    CvFLAGS(MUTABLE_CV(sv)) &= ~CVf_LOCKED;
 			else
-			    CvFLAGS((CV*)sv) |= CVf_LOCKED;
+			    CvFLAGS(MUTABLE_CV(sv)) |= CVf_LOCKED;
 			continue;
 		    }
 		    break;
 		case 'h':
 		    if (memEQ(name, "method", 6)) {
 			if (negated)
-			    CvFLAGS((CV*)sv) &= ~CVf_METHOD;
+			    CvFLAGS(MUTABLE_CV(sv)) &= ~CVf_METHOD;
 			else
-			    CvFLAGS((CV*)sv) |= CVf_METHOD;
+			    CvFLAGS(MUTABLE_CV(sv)) |= CVf_METHOD;
 			continue;
 		    }
 		    break;
@@ -147,30 +148,29 @@ modify_SV_attributes(pTHX_ SV *sv, SV **retlist, SV **attrlist, int numattrs)
 
 XS(XS_attributes_bootstrap)
 {
+    dVAR;
     dXSARGS;
-    PERL_UNUSED_ARG(cv);
 
     if( items > 1 )
-        Perl_croak(aTHX_ "Usage: attributes::bootstrap $module");
+	croak_xs_usage(cv, "$module");
 
-    newXS("attributes::_modify_attrs",	XS_attributes__modify_attrs,	(char *)file);
-    newXSproto("attributes::_guess_stash", XS_attributes__guess_stash, (char *)file, "$");
-    newXSproto("attributes::_fetch_attrs", XS_attributes__fetch_attrs, (char *)file, "$");
-    newXSproto("attributes::reftype",	XS_attributes_reftype,	(char *)file, "$");
+    newXS("attributes::_modify_attrs",	XS_attributes__modify_attrs,	file);
+    newXSproto("attributes::_guess_stash", XS_attributes__guess_stash, file, "$");
+    newXSproto("attributes::_fetch_attrs", XS_attributes__fetch_attrs, file, "$");
+    newXSproto("attributes::reftype",	XS_attributes_reftype,	file, "$");
 
     XSRETURN(0);
 }
 
 XS(XS_attributes__modify_attrs)
 {
+    dVAR;
     dXSARGS;
     SV *rv, *sv;
-    PERL_UNUSED_ARG(cv);
 
     if (items < 1) {
 usage:
-	Perl_croak(aTHX_
-		   "Usage: attributes::_modify_attrs $reference, @attributes");
+	croak_xs_usage(cv, "@attributes");
     }
 
     rv = ST(0);
@@ -185,15 +185,14 @@ usage:
 
 XS(XS_attributes__fetch_attrs)
 {
+    dVAR;
     dXSARGS;
     SV *rv, *sv;
     cv_flags_t cvflags;
-    PERL_UNUSED_ARG(cv);
 
     if (items != 1) {
 usage:
-	Perl_croak(aTHX_
-		   "Usage: attributes::_fetch_attrs $reference");
+	croak_xs_usage(cv, "$reference");
     }
 
     rv = ST(0);
@@ -204,16 +203,14 @@ usage:
 
     switch (SvTYPE(sv)) {
     case SVt_PVCV:
-	cvflags = CvFLAGS((CV*)sv);
+	cvflags = CvFLAGS((const CV *)sv);
 	if (cvflags & CVf_LOCKED)
 	    XPUSHs(newSVpvs_flags("locked", SVs_TEMP));
-#ifdef CVf_LVALUE
 	if (cvflags & CVf_LVALUE)
 	    XPUSHs(newSVpvs_flags("lvalue", SVs_TEMP));
-#endif
 	if (cvflags & CVf_METHOD)
 	    XPUSHs(newSVpvs_flags("method", SVs_TEMP));
-        if (GvUNIQUE(CvGV((CV*)sv)))
+        if (GvUNIQUE(CvGV((const CV *)sv)))
 	    XPUSHs(newSVpvs_flags("unique", SVs_TEMP));
 	break;
     case SVt_PVGV:
@@ -229,15 +226,14 @@ usage:
 
 XS(XS_attributes__guess_stash)
 {
+    dVAR;
     dXSARGS;
     SV *rv, *sv;
     dXSTARG;
-    PERL_UNUSED_ARG(cv);
 
     if (items != 1) {
 usage:
-	Perl_croak(aTHX_
-		   "Usage: attributes::_guess_stash $reference");
+	croak_xs_usage(cv, "$reference");
     }
 
     rv = ST(0);
@@ -247,7 +243,7 @@ usage:
     sv = SvRV(rv);
 
     if (SvOBJECT(sv))
-	sv_setpv(TARG, HvNAME_get(SvSTASH(sv)));
+	sv_setpvn(TARG, HvNAME_get(SvSTASH(sv)), HvNAMELEN_get(SvSTASH(sv)));
 #if 0	/* this was probably a bad idea */
     else if (SvPADMY(sv))
 	sv_setsv(TARG, &PL_sv_no);	/* unblessed lexical */
@@ -262,14 +258,14 @@ usage:
 		stash = CvSTASH(sv);
 	    break;
 	case SVt_PVGV:
-	    if (isGV_with_GP(sv) && GvGP(sv) && GvESTASH((GV*)sv))
-		stash = GvESTASH((GV*)sv);
+	    if (isGV_with_GP(sv) && GvGP(sv) && GvESTASH(MUTABLE_GV(sv)))
+		stash = GvESTASH(MUTABLE_GV(sv));
 	    break;
 	default:
 	    break;
 	}
 	if (stash)
-	    sv_setpv(TARG, HvNAME_get(stash));
+	    sv_setpvn(TARG, HvNAME_get(stash), HvNAMELEN_get(stash));
     }
 
     SvSETMAGIC(TARG);
@@ -278,15 +274,14 @@ usage:
 
 XS(XS_attributes_reftype)
 {
+    dVAR;
     dXSARGS;
     SV *rv, *sv;
     dXSTARG;
-    PERL_UNUSED_ARG(cv);
 
     if (items != 1) {
 usage:
-	Perl_croak(aTHX_
-		   "Usage: attributes::reftype $reference");
+	croak_xs_usage(cv, "$reference");
     }
 
     rv = ST(0);
